@@ -54,7 +54,9 @@ gboolean            tracker_db_manager_init                   (TrackerDBManagerF
                                                                gboolean              *first_time,
                                                                gboolean               shared_cache,
                                                                guint                  select_cache_size,
-                                                               guint                  update_cache_size);
+                                                               guint                  update_cache_size,
+                                                               TrackerBusyCallback    busy_callback,
+                                                               gpointer               busy_user_data);
 void                tracker_db_manager_shutdown               (void);
 void                tracker_db_manager_remove_all             (gboolean               rm_journal);
 void                tracker_db_manager_optimize               (void);
@@ -65,6 +67,7 @@ void                tracker_db_manager_move_to_temp           (void);
 void                tracker_db_manager_restore_from_temp      (void);
 void                tracker_db_manager_init_locations         (void);
 gboolean            tracker_db_manager_has_enough_space       (void);
+void                tracker_db_manager_remove_version_file    (void);
 
 TrackerDBManagerFlags
                     tracker_db_manager_get_flags              (guint *select_cache_size,
