@@ -151,6 +151,7 @@ typedef enum {
 typedef enum {
 	FILE_TYPE_INVALID,
 	FILE_TYPE_PPTX,
+	FILE_TYPE_PPSX,
 	FILE_TYPE_DOCX,
 	FILE_TYPE_XLSX
 } MsOfficeXMLFileType;
@@ -186,6 +187,7 @@ static TrackerExtractData data[] = {
 	{ "application/vnd.ms-*",          extract_msoffice },
 	/* MSoffice2007*/
 	{ "application/vnd.openxmlformats-officedocument.presentationml.presentation", extract_msoffice_xml },
+	{ "application/vnd.openxmlformats-officedocument.presentationml.slideshow",    extract_msoffice_xml },
 	{ "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",         extract_msoffice_xml },
 	{ "application/vnd.openxmlformats-officedocument.wordprocessingml.document",   extract_msoffice_xml },
 	{ NULL, NULL }
@@ -645,6 +647,11 @@ extract_powerpoint_content (GsfInfile *infile,
 	GString *all_texts = NULL;
 	gsf_off_t last_document_container;
 
+	/* If no content requested, return */
+	if (max_bytes == 0) {
+		return NULL;
+	}
+
 	stream = gsf_infile_child_by_name (infile, "PowerPoint Document");
 
 	if (is_encrypted) {
@@ -799,6 +806,11 @@ extract_msword_content (GsfInfile *infile,
 	gint text_buffer_size = 0;
 	gsize n_bytes_remaining;
 
+	/* If no content requested, return */
+	if (n_bytes == 0) {
+		return NULL;
+	}
+
 	document_stream = gsf_infile_child_by_name (infile, "WordDocument");
 	if (document_stream == NULL) {
 		return NULL;
@@ -1425,6 +1437,11 @@ extract_excel_content (GsfInfile *infile,
 	guint saved_offset;
 	gsize n_bytes_remaining = n_bytes;
 
+	/* If no content requested, return */
+	if (n_bytes == 0) {
+		return NULL;
+	}
+
 	stream = gsf_infile_child_by_name (infile, "Workbook");
 
 	if (!stream) {
@@ -1685,9 +1702,15 @@ extract_msoffice (const gchar          *uri,
 		content = extract_msword_content (infile, max_bytes, &is_encrypted);
 	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.ms-powerpoint") == 0) {
 		/* PowerPoint file */
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nfo:Presentation");
+
 		content = extract_powerpoint_content (infile, max_bytes, &is_encrypted);
 	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.ms-excel") == 0) {
 		/* Excel File */
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nfo:Spreadsheet");
+
 		content = extract_excel_content (infile, max_bytes, &is_encrypted);
 	} else {
 		g_message ("Mime type was not recognised:'%s'", mime_used);
@@ -1798,6 +1821,7 @@ xml_start_element_handler_text_data (GMarkupParseContext  *context,
 		break;
 
 	case FILE_TYPE_PPTX:
+	case FILE_TYPE_PPSX:
 		info->tag_type = MS_OFFICE_XML_TAG_SLIDE_TEXT;
 		break;
 
@@ -2150,6 +2174,7 @@ xml_start_element_handler_content_types (GMarkupParseContext  *context,
 		break;
 
 	case FILE_TYPE_PPTX:
+	case FILE_TYPE_PPSX:
 		if ((g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-officedocument.presentationml.slide+xml") == 0) ||
 		    (g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-officedocument.drawingml.diagramData+xml") == 0)) {
 			xml_read (info, part_name + 1, MS_OFFICE_XML_TAG_DOCUMENT_TEXT_DATA);
@@ -2215,6 +2240,8 @@ extract_msoffice_xml (const gchar          *uri,
 		file_type = FILE_TYPE_DOCX;
 	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.openxmlformats-officedocument.presentationml.presentation") == 0) {
 		file_type = FILE_TYPE_PPTX;
+	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.openxmlformats-officedocument.presentationml.slideshow") == 0) {
+		file_type = FILE_TYPE_PPSX;
 	} else if (g_ascii_strcasecmp (mime_used, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet") == 0) {
 		file_type = FILE_TYPE_XLSX;
 	} else {
