@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -25,13 +25,13 @@
 #include <gio/gio.h>
 
 #include <libtracker-common/tracker-common.h>
-#include <libtracker-db/tracker-db.h>
 
+#include <libtracker-data/tracker-data-backup.h>
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-data-query.h>
 #include <libtracker-data/tracker-data-update.h>
+#include <libtracker-data/tracker-data.h>
 #include <libtracker-data/tracker-sparql-query.h>
-#include <libtracker-data/tracker-data-backup.h>
 
 static gint backup_calls = 0;
 static GMainLoop *loop = NULL;
@@ -85,12 +85,12 @@ check_content_in_db (gint expected_instances, gint expected_relations)
  * Run again the queries
  */
 static void
-test_backup_and_restore (void)
+test_backup_and_restore_helper (gboolean journal)
 {
 	gchar  *data_prefix, *data_filename, *backup_filename, *db_location, *meta_db;
 	GError *error = NULL;
 	GFile  *backup_file;
-	gchar *test_schemas[4] = { NULL, NULL, NULL, NULL };
+	gchar *test_schemas[5] = { NULL, NULL, NULL, NULL, NULL };
 
 	db_location = g_build_path (G_DIR_SEPARATOR_S, g_get_current_dir (), "tracker", NULL);
 	data_prefix = g_build_path (G_DIR_SEPARATOR_S, 
@@ -102,18 +102,19 @@ test_backup_and_restore (void)
 	 */ 
 	test_schemas[0] = g_build_path (G_DIR_SEPARATOR_S, TOP_SRCDIR, "tests", "libtracker-data", "ontologies", "20-dc", NULL);
 	test_schemas[1] = g_build_path (G_DIR_SEPARATOR_S, TOP_SRCDIR, "tests", "libtracker-data", "ontologies", "31-nao", NULL);
-	test_schemas[2] = data_prefix;
+	test_schemas[2] = g_build_path (G_DIR_SEPARATOR_S, TOP_SRCDIR, "tests", "libtracker-data", "ontologies", "90-tracker", NULL);
+	test_schemas[3] = data_prefix;
+
+	tracker_db_journal_set_rotating (FALSE, G_MAXSIZE, NULL);
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           (const gchar **) test_schemas,
-	                           NULL, FALSE);
+	                           NULL, FALSE, 100, 100, NULL, NULL, NULL);
 
 	/* load data set */
 	data_filename = g_strconcat (data_prefix, ".data", NULL);
 	if (g_file_test (data_filename, G_FILE_TEST_IS_REGULAR)) {
-		tracker_data_begin_db_transaction ();
 		tracker_turtle_reader_load (data_filename, &error);
-		tracker_data_commit_db_transaction ();
 		g_assert_no_error (error);
 	} else {
 		g_assert_not_reached ();
@@ -146,27 +147,47 @@ test_backup_and_restore (void)
 	g_unlink (meta_db);
 	g_free (meta_db);
 
-	meta_db = g_build_path (G_DIR_SEPARATOR_S, db_location, "data", "tracker-store.journal", NULL);
-	g_unlink (meta_db);
-	g_free (meta_db);
+	if (!journal) {
+		meta_db = g_build_path (G_DIR_SEPARATOR_S, db_location, "data", "tracker-store.journal", NULL);
+		g_unlink (meta_db);
+		g_free (meta_db);
+	}
 
 	meta_db = g_build_path (G_DIR_SEPARATOR_S, db_location, "data", ".meta.isrunning", NULL);
 	g_unlink (meta_db);
 	g_free (meta_db);
 
+	tracker_db_journal_set_rotating (FALSE, G_MAXSIZE, NULL);
+
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           (const gchar **) test_schemas,
-	                           NULL, FALSE);
+	                           NULL, FALSE, 100, 100, NULL, NULL, NULL);
 	check_content_in_db (0, 0);
 
 	test_schemas[0] = data_prefix;
-	tracker_data_backup_restore (backup_file, backup_finished_cb, NULL, NULL, (const gchar **) test_schemas);
+	tracker_data_backup_restore (backup_file, backup_finished_cb, NULL, NULL, (const gchar **) test_schemas, NULL, NULL);
 	check_content_in_db (3, 1);
 
 	g_free (test_schemas[0]);
 	g_free (test_schemas[1]);
 
 	g_assert_cmpint (backup_calls, ==, 2);
+
+	tracker_data_manager_shutdown ();
+}
+
+static void
+test_backup_and_restore (void)
+{
+	test_backup_and_restore_helper (FALSE);
+	backup_calls = 0;
+}
+
+static void
+test_journal_then_backup_and_restore (void)
+{
+	test_backup_and_restore_helper (TRUE);
+	backup_calls = 0;
 }
 
 int
@@ -192,14 +213,18 @@ main (int argc, char **argv)
 
 	g_free (current_dir);
 
+	g_test_add_func ("/tracker/libtracker-data/backup/journal_then_save_and_restore",
+	                 test_journal_then_backup_and_restore);
+
 	g_test_add_func ("/tracker/libtracker-data/backup/save_and_restore",
 	                 test_backup_and_restore);
+
 	/* run tests */
 	result = g_test_run ();
 
 	/* clean up */
 	g_print ("Removing temporary data\n");
-	g_spawn_command_line_async ("rm -R tracker/", NULL);
+	g_spawn_command_line_sync ("rm -R tracker/", NULL, NULL, NULL, NULL);
 
 	return result;
 }
